###############################################################################
##
##  Copyright (C) 2014 Coastform Systems Ltd.
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU Affero General Public License, version 3,
##  as published by the Free Software Foundation.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
##  GNU Affero General Public License for more details.
##
##  You should have received a copy of the GNU Affero General Public License
##  along with this program. If not, see <http://www.gnu.org/licenses/>.
##
###############################################################################

import sys
import fdb
import string
import os
import shutil

from time import localtime, strftime


from twisted.internet.defer import inlineCallbacks

from autobahn.twisted.util import sleep
from autobahn.twisted.wamp import ApplicationSession
from autobahn.wamp.exception import ApplicationError



class AppSession(ApplicationSession):

   def __init__(self, config):
      self.strExtra_WebPath_Rel = config.extra.get('webpath')
      self.strExtra_DbPath = config.extra.get('databasepath')
      self.strExtra_DbUsername = config.extra.get('databaseusername')
      self.strExtra_DbPassword = config.extra.get('databasepassword')
      
      ApplicationSession.__init__(self, config)

   def ProcessExtraConfig(self):
      str_webPath = os.path.abspath(self.strExtra_WebPath_Rel)
      self.m_str_webPath = str_webPath
      self.m_str_Path_TempImages = os.path.join(str_webPath, "tempimages")
      
      self.m_str_DbPath = os.path.abspath(self.strExtra_DbPath)
   
   
   def Db_SetupTrigger(self):
         cur = self.con.cursor()
         str_CreateTrigger = """CREATE OR ALTER TRIGGER POST_USER_LOG FOR "Log" ACTIVE """
         str_CreateTrigger = str_CreateTrigger + """
         AFTER INSERT OR UPDATE POSITION 0
         AS
         BEGIN
            IF( NEW."TagID" is not null AND NEW."ReaderID" is not null AND NEW."UserID" is not null AND (NEW."EventType"='A' OR NEW."EventType"='P')) THEN
               post_event 'new_log_entry' ;
         END"""
         cur.execute(str_CreateTrigger)
         self.con.commit()
         self.eventConduit = self.con.event_conduit(['new_log_entry'])
         self.eventConduit.flush()
         
   
   def OpenDatabase(self):
      print "Db path: " +self.m_str_DbPath
      DsnPath = 'localhost:' +self.m_str_DbPath
      self.con = fdb.connect(dsn=DsnPath, user=self.strExtra_DbUsername, password=self.strExtra_DbPassword, sql_dialect=3, charset='WIN1252')
         
   def CopyImageToWebTemp(self, str_ImagePath_FromDB):
      str_WebImgPath = ""
      try:
         str_path, str_filename = os.path.split(str_ImagePath_FromDB)
         str_DestFileName = os.path.join(self.m_str_Path_TempImages, str_filename)
         shutil.copyfile(str_ImagePath_FromDB, str_DestFileName)
         str_WebImgPath = os.path.relpath(str_DestFileName, self.m_str_webPath)
      except:
         str_WebImgPath = 'missing-image.jpg'      
         print "Warning: Image " +str_ImagePath_FromDB +" is missing or cannot be copied to " +str_DestFileName

        
      return str_WebImgPath
      

   def Db_GetLatestUserActivity(self):
      cur = self.con.cursor()
      strSql = 'SELECT FIRST 1 "Timestamp", "UserID", "ReaderID" FROM "Log"      WHERE "TagID" is not null AND "ReaderID" is not null AND "UserID" is not null AND ("EventType"=\'A\' OR "EventType"=\'P\')            ORDER BY "Timestamp" DESC'
      cur.execute(strSql)
      SqlResult = cur.fetchone()
      [TimeStamp, UserID, ReaderID] = SqlResult
#      print "UserID:", UserID

      strSql = 'SELECT "Direction" FROM "Readers"      WHERE "ReaderID"= ' +str(ReaderID)
      cur.execute(strSql)
      SqlResult = cur.fetchone()
      [StatusNum] = SqlResult
      
      strSql = 'SELECT "FirstName", "LastName", "PhotoIDPathname" FROM "Users" WHERE "UserID" = ' +str(UserID)
      cur.execute(strSql)
      SqlResult = cur.fetchone()
      [FirstName, LastName, PhotoIDPathname] = SqlResult
      strTime = strftime("%a, %d %b %Y %H:%M:%S", localtime(TimeStamp))
      
      if(StatusNum == 1):
         str_Presence = "IN"
      elif(StatusNum == 2):
         str_Presence = "OUT"
      else:
         str_Presence = "Presence unknown"
         
      if PhotoIDPathname:
         PhotoIDPathname = self.CopyImageToWebTemp(PhotoIDPathname)
         PhotoIDPathname = PhotoIDPathname.replace("\\", "/")
      else:
         print "No photoID"
      
#      print FirstName, LastName, PhotoIDPathname
      
      ## PUBLISH an event
      ##
      SubsTopic = 'coastform.eousermon.useractivity'
      FirstName = FirstName+' '+LastName
      DoorName = "UnknownDoor"
      info = (FirstName, PhotoIDPathname, strTime, DoorName, str_Presence)
      self.publish(SubsTopic, info)

     
   @inlineCallbacks
   def onJoin(self, details):
  
      def onhello(msg):
         self.Db_GetLatestUserActivity()

      self.ProcessExtraConfig()
      self.OpenDatabase()
      yield self.Db_GetLatestUserActivity()
      self.Db_SetupTrigger()
         
      sub = yield self.subscribe(onhello, 'com.eousermon.onhello')

      ## Check .. forever
      ##
      while True:

         WaitResult = yield self.eventConduit.wait(1)
#         print WaitResult
         NewLogCount = WaitResult['new_log_entry']
         if(NewLogCount > 0):
            FlushResult = self.eventConduit.flush()
            self.Db_GetLatestUserActivity()
         
         yield sleep(1)
